{ ---------------------------------------------------------------------- }
{ Unit     : UTurtle                                                     }
{ Datum    : 15.02.2002                                                  }
{ Autor    : Gerhard Rhner                                              }
{ Copyright: (c) Gerhard Rhner                                          }
{ ---------------------------------------------------------------------- }
{ Zusammenfassung                                                        }
{ ---------------                                                        }
{ Diese Unit stellt eine Turtle-Komponente fr die Grafikprogrammierung  }
{ zur Verfgung.                                                         }
{                                                                        }
{ ---------------------------------------------------------------------- }
{ Modul-Historie                                                         }
{ --------------                                                         }
{ 1.00 10.11.89 Urfassung in Anlehnung an AppleGraf konzipiert           }
{ 1.01 17.11.89 wesentliche Module fertiggestellt                        }
{ 1.02 16.02.90 Umlaute entfernt: GRAFTABL laden! Show- und HideCursor   }
{ 1.03 14.09.90 Entfernung AppleGraf spezifischer Elemente               }
{ 1.05 15.09.90 Leave- und EnterGraphic                                  }
{ 1.06 26.10.90 generelle berarbeitung, Get/Set-Cursor                  }
{ 1.07 02.11.90 Rckkehr zu blichem Koordinatensystem                   }
{ 1.08 16.11.90 WaitKey                                                  }
{ 1.09 18.11.90 berarbeitung der Druckroutinen                          }
{ 1.10 26.11.90 GraphMode mit HideCursor, TextMode mit ShowCursor        }
{ 1.11 26.04.95 berarbeitung fr VGA-Grafik                             }
{ 1.12 08.05.95 Inverse Darstellung                                      }
{ 2.00 28.10.97 Erste Fassung fr Delphi, als Unit, auf Turtle reduziert }
{ 2.01 19.01.98 Realisierung als Komponente                              }
{ 2.02 19.09.98 Ergnzung der Methode Clear                              }
{ 2.03 27.09.98 Ergnzung der Methode Print und des Ereignisses OnMouseUp}
{ 2.04 07.10.98 PrintSize und DrawDynamic                                }
{ 2.05 15.02.02 Init ergnzt, MouseUp setzt Winkel auf 0                 }
{ ---------------------------------------------------------------------- }

unit UTurtle;

interface

uses
  WinTypes, WinProcs, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Printers;

type
  TTurtle = class(TImage)
  private
    Fx: Double;
    Fy: Double;
    Fw: Double;
    FDynamic: Boolean;
    FPrint: Integer;
    PrintBitmap: TBitmap;
    procedure WTurtleMod360;
    function  GetColor: TColor;
    procedure SetColor(Farbe: TColor);
    function  GetBackColor: TColor;
    procedure SetBackColor(Farbe: TColor);
  protected
    constructor Create(AOwner: TComponent); override;
    procedure TurtleMouseUp(Sender: TObject;
                Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure SetBounds(Left, Top, Width, Height: Integer); override;
  public
    procedure Init;
    procedure TurnTo (Angle: Real);
    procedure Turn   (Angle: Real);
    procedure MoveTo (x, y  : Real);
    procedure Move   (Length: Real);
    procedure DrawTo (x, y  : Real);
    procedure Draw   (Length: Real);
    procedure Clear;
    procedure Print;
  published
    property XPos: Double read Fx write Fx;
    property YPos: Double read Fy write Fy;
    property Angle: Double read Fw write Fw;
    property Color: TColor read GetColor write SetColor;
    property BackColor: TColor read GetBackColor write SetBackColor;
    property PrintSize: Integer read FPrint write FPrint default 4;
    property DrawDynamic: Boolean read FDynamic write FDynamic;
  end;

procedure Register;

{ --- Implementierung ---------------------------------------------------------}

implementation

{ --- private Routinen --------------------------------------------------------}

const PIdurch180 = 0.0174533;

constructor TTurtle.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  OnMouseUp:= TurtleMouseUp;
  Init;
  Cursor:= crCross;
  FPrint:= 4;
  PrintBitmap:= TBitmap.Create;
end;

{ --- Die Turtle in die Mitte setzen und nach rechts schauen lassen -----------}

procedure TTurtle.Init;
begin
  MoveTo(Width div 2, Height div 2);
  TurnTo(0);
end;

{ --- Die Turtle an der Klickstelle positionieren -----------------------------}

procedure TTurtle.TurtleMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  MoveTo(x,y);
  TurnTo(0);
  Canvas.Pixels[x,y]:= Color;
end;

{ --- Grennderungen der Turtle beim Drucken bercksichtigen ----------------}

procedure TTurtle.SetBounds(Left, Top, Width, Height: Integer);
begin
  inherited SetBounds(Left, Top, Width, Height);
  XPos:= Width div 2;
  YPos:= Height div 2;
  If Assigned(PrintBitmap) then begin
    PrintBitmap.Height:= FPrint*Height;
    PrintBitmap.Width:= FPrint*Width;
  end;
  If Assigned(Picture.BitMap) then begin
    Picture.Bitmap.Height:= Height;
    Picture.Bitmap.Width:= Width;
  end;
end;

{ --- reduziert den Angle auf den Bereich 0..359,9999 ----------------- }

procedure TTurtle.WTurtleMod360;
begin
  while Fw >= 360 do Fw:= Fw - 360;
  while Fw <    0 do Fw:= Fw + 360;
end;

{ --- setzt die Farbe der Turtle --------------------------------------------- }

procedure TTurtle.SetColor (Farbe: TColor);
begin
  Canvas.Pen.Color:= Farbe;
end;

{ --- liefert die Farbe der Turtle ------------------------------------------- }

function TTurtle.GetColor: TColor;
begin
  Result:= Canvas.Pen.Color;
end;


{ --- setzt die Hintergrundfarbe der Turtle ---------------------------------- }

procedure TTurtle.SetBackColor (Farbe: TColor);
begin
  Canvas.Brush.Color:= Farbe;
  if csDesigning in ComponentState then  { visuelle Kontrolle im Entwurf }
    Canvas.FillRect(Rect(0,0,Width, Height))
end;

{ --- liefert die Hintergrundfarbe der Turtle ---------------------------------}

function TTurtle.GetBackColor: TColor;
begin
  Result:= Canvas.Brush.Color;
end;

{ --- stellt die Turtle auf einen absoluten Angle --------------------- }

procedure TTurtle.TurnTo (Angle: Real);
begin
  Fw:= Angle; WTurtleMod360;
end;

{ --- dreht die Turtle um einen Angle --------------------------------- }

procedure TTurtle.Turn (Angle: Real);
begin
  Fw:= Fw + Angle; WTurtleMod360;
end;

{ --- bewegt die Turtle zum Punkt P(x/y) ------------------------------- }

procedure TTurtle.MoveTo (x, y: Real);
begin
  Fx:= x; Fy:= y;
end;

{ --- bewegt die Turtle um eine Strecke angegebener Length ------------- }

procedure TTurtle.Move (Length: Real);
begin
  Fx:= Fx + Length * Cos (Fw * PIdurch180);
  Fy:= Fy - Length * Sin (Fw * PIdurch180);
end;

{ --- die Turtle zeichnet eine Strecke zum Punkt P(x/y) ---------------- }

procedure TTurtle.DrawTo (x, y: Real);
begin
  Canvas.MoveTo(Round(Fx), Round(Fy));
  PrintBitmap.Canvas.MoveTo(Round(FPrint*Fx), Round(FPrint*Fy));
  Fx:= x; Fy:= y;
  Canvas.LineTo(Round(Fx), Round(Fy));
  PrintBitmap.Canvas.LineTo(Round(FPrint*Fx), Round(FPrint*Fy));
  If FDynamic then Repaint;
end;

{ --- die Turtle zeichnet eine Strecke angegebener Length -------------- }

procedure TTurtle.Draw (Length: Real);
begin
  DrawTo (Fx + Length * Cos (Fw * PIdurch180),
          Fy - Length * Sin (Fw * PIdurch180));
end;

{ --- Lsche den Arbeitsbereich der Turtle ------------------------------------}

procedure TTurtle.Clear;
begin
  Canvas.FillRect(Rect(0, 0, Width, Height));
  PrintBitmap.Canvas.FillRect(Rect(0,0,FPrint*Width,FPrint*Height));
end;

{ --- Drucke das Bild der Turtle ----------------------------------------------}

procedure TTurtle.Print;
  Var DBx, DBY, dx, dy: Integer;
begin
  if Width > Height then Printer.Orientation:= poLandscape;
  DBx:= PrintBitmap.Width;
  DBy:= PrintBitmap.Height;
  dx:= (Printer.PageWidth - DBx) div 2;
  dy:= (Printer.PageHeight - DBy) div 2;
  Printer.BeginDoc;
  Printer.Canvas.CopyRect(Rect(dx, dy, DBx + dx, DBy + dy), PrintBitmap.Canvas, Rect(0, 0, DBx, DBy));
  Printer.EndDoc;
  Printer.Orientation:= poPortrait;
end;

{--- Registriere die Komponente auf der Komponentenleiste 'Beispiele' ---------}

procedure Register;
begin
  RegisterComponents('Eigene', [TTurtle]);
end;

end.

{ --- Ende der Unit UTurtle.Pas ---------------------------------------- }

